@@ -43,8 +43,6 @@
 #define DOUBLE_EQUAL(x,y) (((x)>(y)-DBL_EPSILON) && ((x)<(y)+DBL_EPSILON))
 #define FLOAT_EQUAL(x,y) (((x)>(y)-FLT_EPSILON) && ((x)<(y)+FLT_EPSILON))
 
-static TRIO_CONST char rcsid[] = "@(#)$Id: regression.c,v 1.59 2006/10/22 12:47:32 breese Exp $";
-
 #if defined(TRIO_EMBED_NAN)
 # include "trionan.c"
 #endif
@@ -103,12 +101,31 @@ TRIO_ARGS4((file, line, expected, got),
 /*************************************************************************
  *
  */
-int
+static int
+Compare
+TRIO_ARGS4((file, line, expected, got),
+	   TRIO_CONST char *file,
+	   int line,
+	   TRIO_CONST char *expected,
+	   TRIO_CONST char *got)
+{
+  if (!trio_equal_case(expected, got))
+    {
+      Report(file, line, expected, got);
+      return 1;
+    }
+  return 0;
+}
+
+/*************************************************************************
+ *
+ */
+static int
 Verify
 TRIO_VARGS5((file, line, result, fmt, va_alist),
 	    TRIO_CONST char *file,
 	    int line,
-	    TRIO_CONST char *result,
+	    TRIO_CONST char *expected,
 	    TRIO_CONST char *fmt,
 	    TRIO_VA_DECL)
 {
@@ -122,18 +139,35 @@ TRIO_VARGS5((file, line, result, fmt, va_alist),
     Dump(buffer, rc);
   TRIO_VA_END(args);
 
-  if (!trio_equal_case(result, buffer))
-    {
-      Report(file, line, result, buffer);
-      return 1;
-    }
-  return 0;
+  return Compare(file, line, expected, buffer);
 }
 
 /*************************************************************************
  *
  */
-int
+static int
+VerifyV
+TRIO_ARGS5((file, line, result, fmt, args),
+	    TRIO_CONST char *file,
+	    int line,
+	    TRIO_CONST char *expected,
+	    TRIO_CONST char *fmt,
+	    trio_pointer_t args)
+{
+  int rc;
+  char buffer[4096];
+
+  rc = trio_snprintfv(buffer, sizeof(buffer), fmt, (trio_pointer_t*)args);
+  if (rc < 0)
+    Dump(buffer, rc);
+
+  return Compare(file, line, expected, buffer);
+}
+
+/*************************************************************************
+ *
+ */
+static int
 VerifyReturnValues(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -142,28 +176,32 @@ VerifyReturnValues(TRIO_NOARGS)
   char *expected;
   char buffer[4096];
   char result[4096];
+  char string1[] = "10 10 0123456789";
+  char string2[] = "10 3 012";
+  char string3[] = "10 0 ";
+  char string4[] = "10 0 DO NOT TOUCH";
 
   rc = trio_sprintf(buffer, "%s%n", "0123456789", &count);
   trio_sprintf(result, "%d %d %s", rc, count, buffer);
-  expected = "10 10 0123456789";
+  expected = string1;
   if (!trio_equal_case(result, expected))
     {
       nerrors++;
       Report(__FILE__, __LINE__, expected, result);
     }
-  
+
   rc = trio_snprintf(buffer, sizeof(buffer), "%s%n", "0123456789", &count);
   trio_sprintf(result, "%d %d %s", rc, count, buffer);
-  expected = "10 10 0123456789";
+  expected = string1;
   if (!trio_equal_case(result, expected))
     {
       nerrors++;
       Report(__FILE__, __LINE__, expected, result);
     }
-  
+
   rc = trio_snprintf(buffer, 4, "%s%n", "0123456789", &count);
   trio_sprintf(result, "%d %d %s", rc, count, buffer);
-  expected = "10 3 012";
+  expected = string2;
   if (!trio_equal_case(result, expected))
     {
       nerrors++;
@@ -173,7 +211,7 @@ VerifyReturnValues(TRIO_NOARGS)
   /* The output buffer contains the empty string */
   rc = trio_snprintf(buffer, 1, "%s%n", "0123456789", &count);
   trio_sprintf(result, "%d %d %s", rc, count, buffer);
-  expected = "10 0 ";
+  expected = string3;
   if (!trio_equal_case(result, expected))
     {
       nerrors++;
@@ -184,13 +222,13 @@ VerifyReturnValues(TRIO_NOARGS)
   trio_sprintf(buffer, "DO NOT TOUCH");
   rc = trio_snprintf(buffer, 0, "%s%n", "0123456789", &count);
   trio_sprintf(result, "%d %d %s", rc, count, buffer);
-  expected = "10 0 DO NOT TOUCH";
+  expected = string4;
   if (!trio_equal_case(result, expected))
     {
       nerrors++;
       Report(__FILE__, __LINE__, expected, result);
     }
-  
+
   return nerrors;
 }
 
@@ -199,7 +237,7 @@ VerifyReturnValues(TRIO_NOARGS)
  */
 #define TEST_STRING "0123456789"
 
-int
+static int
 VerifyAllocate(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -231,15 +269,14 @@ VerifyAllocate(TRIO_NOARGS)
   if (string)
     free(string);
 #endif
-  
+
   return nerrors;
 }
 
-
 /*************************************************************************
  *
  */
-int
+static int
 VerifyFormattingStrings(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -257,12 +294,12 @@ VerifyFormattingStrings(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
 VerifyFormattingIntegers(TRIO_NOARGS)
 {
   int nerrors = 0;
   char buffer[256];
-  
+
   /* Integer */
   nerrors += Verify(__FILE__, __LINE__, "Number 42",
 		   "Number %d", 42);
@@ -364,7 +401,7 @@ VerifyFormattingIntegers(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
 VerifyFormattingFloats(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -530,12 +567,20 @@ VerifyFormattingFloats(TRIO_NOARGS)
 		    "%#11.5g", 1e-4);
   nerrors += Verify(__FILE__, __LINE__, "  0.0010000",
 		    "%#11.5g", 1e-3);
+  nerrors += Verify(__FILE__, __LINE__, "  0.0010000",
+		    "%#11.5g", 0.999999e-3);
   nerrors += Verify(__FILE__, __LINE__, "   0.010000",
 		    "%#11.5g", 1e-2);
+  nerrors += Verify(__FILE__, __LINE__, "   0.010000",
+		    "%#11.5g", 0.999999e-2);
   nerrors += Verify(__FILE__, __LINE__, "    0.10000",
 		    "%#11.5g", 1e-1);
+  nerrors += Verify(__FILE__, __LINE__, "    0.10000",
+		    "%#11.5g", 0.999999e-1);
   nerrors += Verify(__FILE__, __LINE__, "     1.0000",
 		    "%#11.5g", 1e0);
+  nerrors += Verify(__FILE__, __LINE__, "     1.0000",
+		    "%#11.5g", 0.999999e0);
   nerrors += Verify(__FILE__, __LINE__, "     10.000",
 		    "%#11.5g", 1e1);
   nerrors += Verify(__FILE__, __LINE__, "     100.00",
@@ -574,13 +619,13 @@ VerifyFormattingFloats(TRIO_NOARGS)
   nerrors += Verify(__FILE__, __LINE__, "3.141500e+03    ",
 		    "%-16e", 3141.5);
   nerrors += Verify(__FILE__, __LINE__, "03.142e+03",
-		    "%010.3e", 3141.5);
+		    "%010.3e", 3141.51);
 #if !defined(TRIO_COMPILER_ANCIENT)
   /* Long double */
   nerrors += Verify(__FILE__, __LINE__, "1.400000",
 		    "%Lf", 1.4L);
 #endif
-  
+
   /* Special cases */
   nerrors += Verify(__FILE__, __LINE__, "1.00",
 		    "%.2f", 0.999);
@@ -599,7 +644,7 @@ VerifyFormattingFloats(TRIO_NOARGS)
 		    "%f", trio_nan());
   nerrors += Verify(__FILE__, __LINE__, "NAN",
 		    "%F", trio_nan());
-  
+
 # if TRIO_FEATURE_HEXFLOAT
   nerrors += Verify(__FILE__, __LINE__, "0x2.ap+4",
 		    "%a", 42.0);
@@ -616,16 +661,69 @@ VerifyFormattingFloats(TRIO_NOARGS)
   nerrors += Verify(__FILE__, __LINE__, "0xb.351c434a98fa8p-148",
 		    "%a", 3.141e-44);
 # endif
-  
+
 #endif /* TRIO_FEATURE_FLOAT */
-  
+
   return nerrors;
 }
 
 /*************************************************************************
  *
  */
-int
+#if TRIO_EXTENSION
+static int number_writer(void *ref)
+{
+  const char *format;
+  int *data;
+
+  format = trio_get_format(ref);
+  if ((format) && trio_equal(format, "integer"))
+    {
+      data = (int*)trio_get_argument(ref);
+      if (data)
+	{
+	  trio_print_int(ref, *data);
+	}
+    }
+  return 0;
+}
+
+#endif
+
+static int
+VerifyFormattingUserDefined(TRIO_NOARGS)
+{
+  int nerrors = 0;
+#if TRIO_EXTENSION
+  void *number_handle;
+  int integer = 123;
+
+  number_handle = trio_register(number_writer, "number");
+
+  /* Old style */
+  nerrors += Verify(__FILE__, __LINE__, "123",
+		    "%<number:integer>", &integer);
+
+  /* New style */
+  nerrors += Verify(__FILE__, __LINE__, "123",
+		    "$<number:integer|%p>", &integer);
+  nerrors += Verify(__FILE__, __LINE__, "123",
+		    "$<integer|%p%p>", number_handle, &integer);
+  nerrors += Verify(__FILE__, __LINE__, "$<integer|123",
+		    "$<integer|%d", 123);
+  nerrors += Verify(__FILE__, __LINE__, "$integer|123>",
+		    "$integer|%d>", 123);
+
+  trio_unregister(number_handle);
+#endif
+
+  return nerrors;
+}
+
+/*************************************************************************
+ *
+ */
+static int
 VerifyFormattingRegression(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -636,6 +734,10 @@ VerifyFormattingRegression(TRIO_NOARGS)
 		    "%e", 0.5);
   nerrors += Verify(__FILE__, __LINE__, "6.000000e-01",
 		    "%e", 0.6);
+
+  /* Calculate the correct number of fractional digits while rounding */
+  nerrors += Verify(__FILE__, __LINE__, "7654321.0",
+		    "%#.8g", 7654320.99);
 #endif
 
   return nerrors;
@@ -644,17 +746,48 @@ VerifyFormattingRegression(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
+VerifyFormattingArgarray(TRIO_NOARGS)
+{
+  int nerrors = 0;
+  trio_pointer_t argarray[4];
+  int value = 42;
+  double number = 123.456;
+  float smallNumber = 123.456;
+  char string[] = "my string";
+
+  argarray[0] = &value;
+  nerrors += VerifyV(__FILE__, __LINE__, "42",
+		     "%d", argarray);
+  argarray[0] = string;
+  nerrors += VerifyV(__FILE__, __LINE__, "my string",
+		     "%s", argarray);
+  argarray[0] = &number;
+  argarray[1] = &smallNumber;
+  nerrors += VerifyV(__FILE__, __LINE__, "123.456000 123.456001",
+		     "%f %hf", argarray);
+
+  return nerrors;
+}
+
+/*************************************************************************
+ *
+ */
+static int
 VerifyFormatting(TRIO_NOARGS)
 {
   int nerrors = 0;
+#if TRIO_FEATURE_SIZE_T || TRIO_FEATURE_SIZE_T_UPPER
   char buffer[256];
+#endif
 
   nerrors += VerifyFormattingStrings();
   nerrors += VerifyFormattingIntegers();
   nerrors += VerifyFormattingFloats();
   nerrors += VerifyFormattingRegression();
-  
+  nerrors += VerifyFormattingUserDefined();
+  nerrors += VerifyFormattingArgarray();
+
   /* Pointer */
   if (sizeof(void *) == 4)
     {
@@ -671,7 +804,7 @@ VerifyFormatting(TRIO_NOARGS)
   /* Nil pointer */
   nerrors += Verify(__FILE__, __LINE__, "Pointer (nil)",
 		   "Pointer %p", NULL);
-  
+
   /* Char width alignment */
   nerrors += Verify(__FILE__, __LINE__, "Char X   .",
 	 "Char %-4c.", 'X');
@@ -698,7 +831,7 @@ VerifyFormatting(TRIO_NOARGS)
 		    "%4$d %3$*8$d %2$.*7$d %1$*6$.*5$d",
 		    123, 1234, 12345, 123456, 5, 6, 7, 8);
 #endif
-  
+
 #if TRIO_FEATURE_SIZE_T_UPPER
   nerrors += Verify(__FILE__, __LINE__, "256",
 		    "%Zd", sizeof(buffer));
@@ -719,7 +852,7 @@ VerifyFormatting(TRIO_NOARGS)
 		    "%m");
 # endif
 #endif
-  
+
 #if TRIO_FEATURE_QUAD
 # if defined(TRIO_COMPILER_SUPPORTS_LL)
   /* This may fail if the preprocessor does not recognize LL */
@@ -761,7 +894,7 @@ VerifyFormatting(TRIO_NOARGS)
   nerrors += Verify(__FILE__, __LINE__, "ffffffff",
 		    "%I16x", -1);
 #endif
-  
+
 #if TRIO_EXTENSION
   nerrors += Verify(__FILE__, __LINE__, "  42   86",
 		    "%!4d %d", 42, 86);
@@ -805,7 +938,7 @@ VerifyFormatting(TRIO_NOARGS)
 		    "%.17Rf", 1.4);
   nerrors += Verify(__FILE__, __LINE__, "39413.8",
 		    "%.30Rf", 39413.80);
-#  if !defined(TRIO_COMPILER_ANCIENT)
+#  if !defined(TRIO_COMPILER_ANCIENT) && !defined(TRIO_DOUBLE_DOUBLE)
   /* Long double */
   nerrors += Verify(__FILE__, __LINE__, "1.4",
 		    "%RLf", 1.4L);
@@ -840,20 +973,20 @@ VerifyFormatting(TRIO_NOARGS)
 		    "%.30RLf", (2.0L/3.0L));
 # endif
 #endif
-  
+
   return nerrors;
 }
 
 /*************************************************************************
  *
  */
-int
+static int
 VerifyErrors(TRIO_NOARGS)
 {
   char buffer[512];
   int rc;
   int nerrors = 0;
-  
+
   /* Error: Invalid argument 1 */
   rc = trio_snprintf(buffer, sizeof(buffer), "%d %r", 42, "text");
 #if TRIO_FEATURE_ERRORCODE
@@ -874,7 +1007,7 @@ VerifyErrors(TRIO_NOARGS)
 #else
   nerrors += (rc != -1);
 #endif
-  
+
   /* Error: Invalid argument 2 */
   rc = trio_snprintf(buffer, sizeof(buffer), "%#");
 #if TRIO_FEATURE_ERRORCODE
@@ -895,7 +1028,7 @@ VerifyErrors(TRIO_NOARGS)
 #else
   nerrors += (rc != -1);
 #endif
-  
+
   /* Error: Invalid argument 3 */
   rc = trio_snprintf(buffer, sizeof(buffer), "%hhhd", 42);
 #if TRIO_FEATURE_ERRORCODE
@@ -916,7 +1049,7 @@ VerifyErrors(TRIO_NOARGS)
 #else
   nerrors += (rc != -1);
 #endif
-  
+
   /* Error: Double reference */
   rc = trio_snprintf(buffer, sizeof(buffer), "hello %1$d %1$d", 31, 32);
 #if TRIO_FEATURE_ERRORCODE
@@ -947,7 +1080,7 @@ VerifyErrors(TRIO_NOARGS)
 #else
   nerrors += (rc != -1);
 #endif
-  
+
   /* Error: Reference gap */
   rc = trio_snprintf(buffer, sizeof(buffer), "%3$d %1$d", 31, 32, 33);
 #if TRIO_FEATURE_ERRORCODE
@@ -978,7 +1111,7 @@ VerifyErrors(TRIO_NOARGS)
 #else
   nerrors += (rc != -1);
 #endif
-  
+
   return nerrors;
 }
 
@@ -986,7 +1119,7 @@ VerifyErrors(TRIO_NOARGS)
  *
  */
 #if TRIO_FEATURE_SCANF
-int
+static int
 VerifyScanningOneInteger
 TRIO_ARGS5((file, line, expected, format, original),
 	   TRIO_CONST char *file,
@@ -997,13 +1130,13 @@ TRIO_ARGS5((file, line, expected, format, original),
 {
   int number;
   char data[512];
-  
+
   trio_snprintf(data, sizeof(data), format, original);
   trio_sscanf(data, format, &number);
   return Verify(file, line, expected, format, number);
 }
 
-int
+static int
 VerifyScanningIntegers(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -1037,7 +1170,7 @@ VerifyScanningIntegers(TRIO_NOARGS)
  *
  */
 #if TRIO_FEATURE_SCANF
-int
+static int
 VerifyScanningOneFloat
 TRIO_ARGS5((file, line, expected, format, original),
 	   TRIO_CONST char *file,
@@ -1048,13 +1181,13 @@ TRIO_ARGS5((file, line, expected, format, original),
 {
   float number;
   char data[512];
-  
+
   trio_snprintf(data, sizeof(data), format, original);
   trio_sscanf(data, format, &number);
   return Verify(file, line, expected, format, number);
 }
 
-int
+static int
 VerifyScanningOneDouble
 TRIO_ARGS5((file, line, expected, format, original),
 	   TRIO_CONST char *file,
@@ -1065,13 +1198,13 @@ TRIO_ARGS5((file, line, expected, format, original),
 {
   double number;
   char data[512];
-  
+
   trio_snprintf(data, sizeof(data), format, original);
   trio_sscanf(data, format, &number);
   return Verify(file, line, expected, format, number);
 }
 
-int
+static int
 VerifyScanningFloats(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -1122,7 +1255,7 @@ VerifyScanningFloats(TRIO_NOARGS)
   nerrors += VerifyScanningOneFloat(__FILE__, __LINE__, "-inf",
 				      "%f", trio_ninf());
 #endif
-  
+
   return nerrors;
 }
 #endif
@@ -1131,7 +1264,7 @@ VerifyScanningFloats(TRIO_NOARGS)
  *
  */
 #if TRIO_FEATURE_SCANF
-int
+static int
 VerifyScanningOneString
 TRIO_ARGS5((file, line, expected, format, original),
 	   TRIO_CONST char *file,
@@ -1142,41 +1275,50 @@ TRIO_ARGS5((file, line, expected, format, original),
 {
   char string[512];
   char data[512];
-  
+
   trio_snprintf(data, sizeof(data), "%s", original);
+  string[0] = 0;
   trio_sscanf(data, format, string);
   return Verify(file, line, expected, "%s", string);
 }
 
-int
+static int
 VerifyScanningStrings(TRIO_NOARGS)
 {
   int nerrors = 0;
+  char string1[] = "";
+  char string2[] = "hello";
+  char string3[] = "hello world";
+  char string4[] = "abcba";
 
+  nerrors += VerifyScanningOneString(__FILE__, __LINE__, "",
+				     "hello", string2);
+  nerrors += VerifyScanningOneString(__FILE__, __LINE__, "",
+				     "", string1);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "hello",
-				     "%s", "hello");
+				     "%s", string2);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "hello",
-				     "%s", "hello world");
+				     "%s", string3);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "hello world",
-				     "%[^\n]", "hello world");
+				     "%[^\n]", string3);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "(nil)",
 				     "%s", NULL);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "hello",
-				     "%20s", "hello");
+				     "%20s", string2);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "he",
-				     "%2s", "hello");
+				     "%2s", string2);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "ab",
-				     "%[ab]", "abcba");
+				     "%[ab]", string4);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "abcba",
-				     "%[abc]", "abcba");
+				     "%[abc]", string4);
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "abcba",
-				     "%[a-c]", "abcba");
+				     "%[a-c]", string4);
 #if TRIO_EXTENSION
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "abcba",
-				     "%[[:alpha:]]", "abcba");
+				     "%[[:alpha:]]", string4);
 #endif
   nerrors += VerifyScanningOneString(__FILE__, __LINE__, "ba",
-				     "%*[ab]c%[^\n]", "abcba");
+				     "%*[ab]c%[^\n]", string4);
 
   return nerrors;
 }
@@ -1186,7 +1328,7 @@ VerifyScanningStrings(TRIO_NOARGS)
  *
  */
 #if TRIO_FEATURE_SCANF
-int
+static int
 VerifyScanningRegression(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -1201,6 +1343,8 @@ VerifyScanningRegression(TRIO_NOARGS)
   long lnumber;
   int number;
   char ch;
+  char buffer[4096];
+  FILE *stream;
 
 #if TRIO_FEATURE_FLOAT
   rc = trio_sscanf("1.5", "%lf%n", &dnumber, &offset);
@@ -1241,6 +1385,59 @@ VerifyScanningRegression(TRIO_NOARGS)
   nerrors += Verify(__FILE__, __LINE__, "1e-06",
 		    "%g", dnumber);
 #endif
+
+  /* Do not overwrite result on matching error */
+  ch = 'a';
+  rc = trio_sscanf("0123456789", "%1[c]", &ch);
+  nerrors += Verify(__FILE__, __LINE__, "a",
+		    "%c", ch);
+
+  /* Scan plus prefix for unsigned integer */
+  rc = trio_sscanf("+42", "%u", &number);
+  nerrors += Verify(__FILE__, __LINE__, "1 42",
+		    "%d %u", rc, number);
+
+  /* Scan minus prefix even for unsigned integer */
+  rc = trio_sscanf("-42", "%u", &number);
+  sprintf(buffer, "1 %u", -42U);
+  nerrors += Verify(__FILE__, __LINE__, buffer,
+		    "%d %u", rc, number);
+
+  /* A scangroup match failure should not bind its argument,
+   * i.e., it shouldn't match the empty string. */
+  sprintf(buffer, "SPQR");
+  rc = trio_sscanf("asdf", "%[c]", buffer);
+  nerrors += Verify(__FILE__, __LINE__, "0 SPQR",
+		    "%d %s", rc, buffer);
+
+  /* Even whitespace scanning shouldn't try to read past EOF */
+  stream = tmpfile();
+  trio_fprintf(stream, "");
+  rewind(stream);
+  rc = trio_fscanf(stream, " ");
+  nerrors += Verify(__FILE__, __LINE__, "0",
+		    "%d", rc);
+  fclose(stream);
+
+  /* Idem, after a successful read */
+  stream = tmpfile();
+  trio_fprintf(stream, "123");
+  rewind(stream);
+  rc = trio_fscanf(stream, "%i ", &number);
+  nerrors += Verify(__FILE__, __LINE__, "1 123",
+		    "%d %i", rc, number);
+  fclose(stream);
+
+  /* The scanner should unget its read-ahead char */
+  stream = tmpfile();
+  trio_fprintf(stream, "123");
+  rewind(stream);
+  trio_fscanf(stream, "%*c");
+  trio_fscanf(stream, "%c", &ch);
+  nerrors += Verify(__FILE__, __LINE__, "2",
+		    "%c", ch);
+  fclose(stream);
+
   return nerrors;
 }
 #endif
@@ -1248,7 +1445,7 @@ VerifyScanningRegression(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
 VerifyScanning(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -1264,7 +1461,7 @@ VerifyScanning(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
 VerifyStrings(TRIO_NOARGS)
 {
   int nerrors = 0;
@@ -1353,7 +1550,7 @@ VerifyStrings(TRIO_NOARGS)
     nerrors++;
     Report0(__FILE__, __LINE__);
   }
-  
+
   /* To upper/lower */
   trio_copy(buffer, "Find me now");
   trio_upper(buffer);
@@ -1397,7 +1594,7 @@ VerifyStrings(TRIO_NOARGS)
     nerrors++;
     Report0(__FILE__, __LINE__);
   }
-  
+
 #endif /* !defined(TRIO_MINIMAL) */
   return nerrors;
 }
@@ -1405,12 +1602,13 @@ VerifyStrings(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
 VerifyDynamicStrings(TRIO_NOARGS)
 {
   int nerrors = 0;
 #if !defined(TRIO_MINIMAL)
   trio_string_t *string;
+  const char no_terminate[5] = { 'h', 'e', 'l', 'l', 'o' };
 
   string = trio_xstring_duplicate("Find me now");
   if (string == NULL) {
@@ -1430,11 +1628,14 @@ VerifyDynamicStrings(TRIO_NOARGS)
     nerrors++;
   if (trio_xstring_match_case(string, "* ME *"))
     nerrors++;
-  
+  if (!trio_xstring_append_max(string, no_terminate, 5) ||
+      !trio_xstring_equal(string, "FIND ME NOW AND AGAINhello"))
+    nerrors++;
+
  error:
   if (string)
     trio_string_destroy(string);
-  
+
 #endif /* !defined(TRIO_MINIMAL) */
   return nerrors;
 }
@@ -1442,14 +1643,14 @@ VerifyDynamicStrings(TRIO_NOARGS)
 /*************************************************************************
  *
  */
-int
+static int
 VerifyNaN(TRIO_NOARGS)
 {
   double ninf_number = trio_ninf();
   double pinf_number = trio_pinf();
   double nan_number = trio_nan();
   int nerrors = 0;
-  
+
   nerrors += Verify(__FILE__, __LINE__, "-1",
 		    "%d", trio_isinf(ninf_number));
   nerrors += Verify(__FILE__, __LINE__, "0",
@@ -1472,41 +1673,43 @@ main(TRIO_NOARGS)
 {
   int nerrors = 0;
 
-  printf("%s\n", rcsid);
-
 #if TRIO_EXTENSION
+  char string1[] = ".";
+  char string2[] = ",";
+  char string3[] = "\3";
+
   /* Override system locale settings */
-  trio_locale_set_decimal_point(".");
-  trio_locale_set_thousand_separator(",");
-  trio_locale_set_grouping("\3");
+  trio_locale_set_decimal_point(string1);
+  trio_locale_set_thousand_separator(string2);
+  trio_locale_set_grouping(string3);
 #endif
 
   printf("Verifying strings\n");
   nerrors += VerifyStrings();
-  
+
   printf("Verifying dynamic strings\n");
   nerrors += VerifyDynamicStrings();
 
   printf("Verifying special quantities\n");
   nerrors += VerifyNaN();
-  
+
   printf("Verifying formatting\n");
   nerrors += VerifyFormatting();
-  
+
   printf("Verifying scanning\n");
   nerrors += VerifyScanning();
-  
+
   printf("Verifying return values\n");
   nerrors += VerifyErrors();
   nerrors += VerifyReturnValues();
-  
+
   printf("Verifying allocation\n");
   nerrors += VerifyAllocate();
 
   if (nerrors == 0)
-    printf("Regression test suceeded\n");
+    printf("Regression test succeeded\n");
   else
     printf("Regression test failed in %d instance(s)\n", nerrors);
-  
+
   return nerrors ? 1 : 0;
 }
